/*
 * A filesystem directory listing accessor.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/types.h>

#include <stdio.h>

#include <e2access/image.h>
#include <e2access/path.h>
#include "fs_directory_listing_accessor.h"



/* Accessor methods. */

unsigned int Fs_directory_listing_accessor::detach()
{
  unsigned int attached = PagingAccessor::detach();

  if (!attached)
  {
    if (_listing)
    {
      _listing->close();
      delete _listing;
      _listing = 0;
    }
  }

  return attached;
}

long Fs_directory_listing_accessor::flush()
{
  return L4_EOK;
}

size_t Fs_directory_listing_accessor::fstat(char *buffer, size_t length)
{
  /* NOTE: Not currently supported. */

  (void) buffer; (void) length;

  return 0;
}

size_t Fs_directory_listing_accessor::read(size_t start, char *buffer, size_t length)
{
  /* Obtain a listing object. */

  if (!_listing)
  {
    _listing = new Fs_directory_listing(_fs, _dir, _flags);

    long err = _listing->start();
    if (err)
      return 0;
  }

  /* Populate the buffer with data from the listing. */

  return _listing->read_into(start, buffer, length);
}

size_t Fs_directory_listing_accessor::write(size_t start, char *buffer, size_t length)
{
  (void) start; (void) buffer; (void) length;

  return 0;
}



/* Special methods. */

long Fs_directory_listing_accessor::create_file(const char *name, uid_t uid,
                                                gid_t gid, mode_t umask,
                                                ext2_ino_t *ino_file)
{
  /* Only create the file if the name does not contain a path separator. */

  if (!path_is_leafname(name))
    return -L4_EIO;

  /* Create the file. */

  errcode_t retval = image_create_file(_fs, _dir, name, umask, uid, gid,
                                       ino_file);
  if (retval)
    return -L4_EIO;

  return L4_EOK;
}
