/*
 * A filesystem object.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <fsserver/resource_server.h>

#include "fs_filesystem.h"
#include "fs_user_filesystem.h"



/* Factory interface definition. */

long Fs_filesystem::getuserfs(ipc_varg_sys_uid_t uid,
                              ipc_varg_sys_gid_t gid,
                              ipc_varg_sys_mode_t umask,
                              l4_cap_idx_t *ref)
{
  /* NOTE: Should be able to employ an automated way of checking tags. */

  if (ipc_varg_word(uid.tag) && ipc_varg_word(gid.tag) && ipc_varg_word(umask.tag))
    return getuserfs(uid.value, gid.value, umask.value, ref);

  return -L4_EINVAL;
}

/* Filesystem interface definition. */

long Fs_filesystem::getuserfs(sys_uid_t uid, sys_gid_t gid, sys_mode_t umask,
                              l4_cap_idx_t *ref)
{
  /* Obtain the user details. */

  user_t user;

  user.uid = uid;
  user.gid = gid;
  user.umask = umask;

  /* Create a user-configured server for the filesystem with a reference to the
     paging object coordinator so that file data can be shared. */

  Fs_user_filesystem *filesystem = new Fs_user_filesystem(_paging, _fs, _devname, user);
  ResourceServer server(filesystem);
  long err = server.start_thread();

  *ref = server.config()->server;
  return err;
}
